"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const Common_1 = __importDefault(require("../functions/Common"));
const ConfigNames_1 = require("../types/ConfigNames");
const SimpleLogger_1 = __importDefault(require("./SimpleLogger"));
class SimpleConfig {
    key;
    path;
    config = JSON.parse("{}");
    constructor(key, path) {
        this.key = key;
        this.path = path;
        this.init();
        this.updateTypes();
    }
    async init() {
        if (!fs_1.default.existsSync(this.path)) {
            await fs_1.default.writeFileSync(this.path, JSON.stringify({}));
        }
        this.loadConfig();
    }
    async updateTypes() {
        let path = process.cwd() + "/src/types/ConfigNames.ts";
        if (!fs_1.default.existsSync(path)) {
            path = process.cwd() + "/node_modules/Foundation/types/ConfigNames.ts";
        }
        if (!fs_1.default.existsSync(path)) {
            SimpleLogger_1.default.log("Failed to save new config typings!");
        }
        let arr = ConfigNames_1.currNames;
        if (arr.includes(this.key))
            return;
        arr.push(this.key);
        let str = `const currNames = [${arr.map(r => `"${r}"`).join(", ")}];`;
        str += `\ntype config = ${arr.map(r => `"${r}"`).join("|")};`;
        str += `\nexport {config, currNames};`;
        fs_1.default.writeFileSync(path, str);
    }
    async loadConfig() {
        const file = fs_1.default.readFileSync(this.path, "utf-8");
        const json = JSON.parse(file);
        if (json == {} || json == null || json == undefined) {
            SimpleLogger_1.default.log("Configuration is empty -" + this.path);
            return this.config = JSON.parse("{}");
        }
        SimpleLogger_1.default.info("Automatically loaded configuration -" + this.path);
        return this.config = json;
    }
    async update(updatedSchema, fileName) {
        this.config = updatedSchema;
        fs_1.default.writeFileSync(this.path, JSON.stringify(updatedSchema, null, 4));
        SimpleLogger_1.default.log(`${this.path} -> Configuration updated by ${fileName}.js`);
        return;
    }
    get(path) {
        let split = path.split(".");
        if (split.length <= 1) {
            return this.config[split.shift()];
        }
        return this.findPath(split);
    }
    isExists(path) {
        return !Common_1.default.isNull(this.get(path));
    }
    findPath(property) {
        var data = this.config;
        for (const r of property) {
            data = data[r];
        }
        return data;
    }
}
exports.default = SimpleConfig;
