"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const SimpleCommand_1 = __importDefault(require("../classes/SimpleCommand"));
const SimpleConfig_1 = __importDefault(require("../classes/SimpleConfig"));
const SimpleEvent_1 = __importDefault(require("../classes/SimpleEvent"));
const SimpleLogger_1 = __importDefault(require("../classes/SimpleLogger"));
// @Exclude-API
async function default_1(app) {
    const files = (await fetchFiles(process.cwd())).filter(r => (r.endsWith("js") || r.endsWith("ts")));
    if (files.length == 0) {
        return SimpleLogger_1.default.log("AutoRegister found 0 files in main directory.");
    }
    const events = [];
    const commands = [];
    for (const filePath of files) {
        if (!fs_1.default.existsSync(filePath))
            continue;
        const file = fs_1.default.readFileSync(filePath, "utf-8");
        if (!file.includes("@AutoRegister") && !file.includes("@Config"))
            continue;
        if (file.includes("@Exclude-API"))
            continue;
        const importFile = await (await Promise.resolve().then(() => __importStar(require(filePath)))).default;
        const Class = new importFile();
        if (Class instanceof SimpleEvent_1.default) {
            events.push(filePath);
        }
        else if (Class instanceof SimpleCommand_1.default) {
            commands.push(filePath);
        }
        else if (Class instanceof SimpleConfig_1.default) {
            app.config.set(Class.key, Class);
        }
    }
    if (events.length !== 0) {
        for (const eventPath of events) {
            if (!fs_1.default.existsSync(eventPath))
                continue;
            const file = (await Promise.resolve().then(() => __importStar(require(eventPath))))?.default;
            const event = new file();
            app.client.on(event.name, event.execute.bind(null, app));
            SimpleLogger_1.default.info("Automatically loaded listener -" + eventPath);
        }
    }
    if (commands.length !== 0) {
        for (const commandPath of commands) {
            if (!fs_1.default.existsSync(commandPath))
                continue;
            const file = (await Promise.resolve().then(() => __importStar(require(commandPath))))?.default;
            const command = new file(app);
            app.commands.set(command.name, command);
            SimpleLogger_1.default.info("Automatically loaded command -" + commandPath);
        }
        const guilds = app.getCommandSettings().registerAllGuilds ?
            app.client.guilds.cache.toJSON().map(r => r.id) :
            app.getCommandSettings()?.guilds ?? [];
        for (const guildId of guilds) {
            const guild = app.client.guilds.cache.get(guildId);
            guild?.commands.set(app.commands.map(r => {
                return {
                    name: r.name,
                    description: r.description,
                    options: r.options
                };
            }));
        }
    }
    return;
}
exports.default = default_1;
async function fetchFiles(path) {
    const baseFilter = ["node_modules", "package-lock.json", "package.json"];
    const baseFilesUnformatted = fs_1.default.readdirSync(path).filter(r => !baseFilter.includes(r));
    const baseFiles = [];
    for (const baseFile of baseFilesUnformatted) {
        baseFiles.push(path + "\\" + baseFile);
    }
    let finalFiles = [];
    for (const baseFile of baseFiles) {
        if (!fs_1.default.existsSync(baseFile))
            continue;
        if (fs_1.default.statSync(baseFile).isDirectory()) {
            finalFiles = finalFiles.concat(await fetchFiles(baseFile));
        }
        else {
            finalFiles.push(baseFile);
        }
    }
    return finalFiles;
}
